@@ -21,6 +21,7 @@ import numbers
 import os
 import shlex
 import sys
+import warnings
 
 try:
     from win32com.client import dynamic
@@ -298,6 +299,10 @@ class SASSessionCOM(object):
         # Store flush result in running log
         self._log += result
 
+        if result.count('ERROR:') > 0:
+           warnings.warn("Noticed 'ERROR:' in LOG, you ought to take a look and see if there was a problem")
+           self._sb.check_error_log = True
+
         return result
 
     def _getfile(self, fname: str, buf: int=2048, decode: bool=False) -> str:
@@ -688,6 +693,9 @@ class SASSessionCOM(object):
 
         tablepath = self._tablepath(table, libref=libref)
 
+        if type(df.index) != pd.RangeIndex:
+           warnings.warn("Note that Indexes are not transferred over as columns. Only actual coulmns are transferred")
+
         columns = []
         formats = {}
         for i, name in enumerate(df.columns):
