@@ -20,6 +20,7 @@ from time import sleep
 import socket as socks
 import tempfile as tf
 import codecs
+import warnings
 
 try:
    import pandas as pd
@@ -58,6 +59,7 @@ class SASconfigIOM:
       self.javaparms = cfg.get('javaparms', '')
       self.lrecl     = cfg.get('lrecl', None)
       self.reconnect = cfg.get('reconnect', True)
+      self.reconuri  = cfg.get('reconuri', None)
       self.logbufsz  = cfg.get('logbufsz', None)
 
       try:
@@ -219,6 +221,13 @@ class SASconfigIOM:
          else:
             self.reconnect = bool(inrecon)
 
+      inruri = kwargs.get('reconuri', None)
+      if inruri is not None:
+         if lock and self.reconuri:
+            print("Parameter 'reconuri' passed to SAS_session was ignored due to configuration restriction.")
+         else:
+            self.reconuri = inruri
+
       inlogsz = kwargs.get('logbufsz', None)
       if inlogsz:
          if inlogsz < 32:
@@ -263,6 +272,7 @@ class SASsessionIOM():
       self.sascfg   = SASconfigIOM(self, **kwargs)
 
       self._startsas()
+      self._sb.reconuri = None
 
    def __del__(self):
       if self.pid:
@@ -374,6 +384,8 @@ Will use HTML5 for this SASsession.""")
       parms += ["-lrecl", str(self.sascfg.lrecl)]
       if self.sascfg.logbufsz is not None:
          parms += ["-logbufsz", str(self.sascfg.logbufsz)]
+      if self.sascfg.reconuri is not None:
+         parms += ["-uri", self.sascfg.reconuri]
       parms += ['']
 
       s = ''
@@ -492,7 +504,7 @@ Will use HTML5 for this SASsession.""")
       self.stdout[0].setblocking(False)
       self.stderr[0].setblocking(False)
 
-      if not zero:
+      if not zero and not self.sascfg.reconuri:
          if not self.sascfg.sspi:
             while len(pw) == 0:
                pw = self.sascfg._prompt("Please enter the password for IOM user "+self.sascfg.omruser+": ", pw=True)
@@ -954,7 +966,7 @@ Will use HTML5 for this SASsession.""")
          except UnicodeDecodeError:
             lstf = lstf.decode(errors='replace')
 
-      logf = logf.decode(errors='replace')
+      logf = logf.decode(errors='replace').replace(chr(12), chr(10))
 
       trip = lstf.rpartition("/*]]>*/")
       if len(trip[1]) > 0 and len(trip[2]) < 200:
@@ -970,6 +982,10 @@ Will use HTML5 for this SASsession.""")
       lstd = lstf.replace(chr(12), chr(10)).replace('<body class="c body">',
                                                     '<body class="l body">').replace("font-size: x-small;",
                                                                                      "font-size:  normal;")
+      if logd.count('ERROR:') > 0:
+         warnings.warn("Noticed 'ERROR:' in LOG, you ought to take a look and see if there was a problem")
+         self._sb.check_error_log = True
+
       self._sb._lastlog = logd
       return dict(LOG=logd, LST=lstd)
 
@@ -1089,8 +1105,10 @@ Will use HTML5 for this SASsession.""")
             if log.count("DISCONNECT") >= 1:
                break
 
-      return log.rstrip("DISCONNECT")
+      res = log.rpartition("DISCONNECT")
+      self._sb.reconuri = res[2].rstrip("END_DISCON")
 
+      return res[0]
 
    def exist(self, table: str, libref: str ="") -> bool:
       """
@@ -1493,6 +1511,9 @@ Will use HTML5 for this SASsession.""")
 
       charlens = {k.upper():v for k,v in charlens.items()}
 
+      if type(df.index) != pd.RangeIndex:
+         warnings.warn("Note that Indexes are not transferred over as columns. Only actual coulmns are transferred")
+
       for name in df.columns:
          colname = str(name)
          input  += "'"+colname+"'n "
@@ -1511,7 +1532,8 @@ Will use HTML5 for this SASsession.""")
                input  += "~ "
             dts.append('C')
             if embedded_newlines:
-               xlate += " '"+colname+"'n = translate('"+colname+"'n, '0A'x, "+lf+");\n"
+               nl     = "'"+'%02x' % ord('\n'.encode(self.sascfg.encoding))+"'x".upper() # for MVS support
+               xlate += " '"+colname+"'n = translate('"+colname+"'n, "+nl+", "+lf+");\n"
                xlate += " '"+colname+"'n = translate('"+colname+"'n, '0D'x, "+cr+");\n"
          else:
             if df.dtypes[name].kind in ('M'):
@@ -1893,7 +1915,12 @@ Will use HTML5 for this SASsession.""")
                    if logf.count(logcodeb) >= 1:
                       bail = True
          done = True
-         self._log += logf.decode(errors='replace')
+
+      logd = logf.decode(errors='replace')
+      self._log += logd
+      if logd.count('ERROR:') > 0:
+         warnings.warn("Noticed 'ERROR:' in LOG, you ought to take a look and see if there was a problem")
+         self._sb.check_error_log = True
 
       if len(r) > 0 or df is None:
          tdf = pd.DataFrame.from_records(r, columns=varlist)
@@ -2124,7 +2151,6 @@ Will use HTML5 for this SASsession.""")
                           if logf.count(logcodeb) >= 1:
                              bail = True
                 done = True
-                self._log += logf.decode(errors='replace')
 
          csv.close()
          df = pd.read_csv(tmpcsv, index_col=idx_col, engine=eng, dtype=dts, **kwargs)
@@ -2150,13 +2176,18 @@ Will use HTML5 for this SASsession.""")
                logf += log
                if logf.count(logcodeb) >= 1:
                   bail = True;
-                  self._log += logf.decode(errors='replace')
 
             if done and bail:
                break
 
          df = pd.read_csv(tmpcsv, index_col=idx_col, engine=eng, dtype=dts, **kwargs)
 
+      logd = logf.decode(errors='replace')
+      self._log += logd
+      if logd.count('ERROR:') > 0:
+         warnings.warn("Noticed 'ERROR:' in LOG, you ought to take a look and see if there was a problem")
+         self._sb.check_error_log = True
+ 
       if tmpdir:
          tmpdir.cleanup()
       else:
@@ -2425,7 +2456,6 @@ Will use HTML5 for this SASsession.""")
                           if logf.count(logcodeb) >= 1:
                              bail = True
                 done = True
-                self._log += logf.decode(errors='replace')
 
          csv.close()
       else:
@@ -2450,11 +2480,16 @@ Will use HTML5 for this SASsession.""")
                logf += log
                if logf.count(logcodeb) >= 1:
                   bail = True;
-                  self._log += logf.decode(errors='replace')
 
             if done and bail:
                break
 
+      logd = logf.decode(errors='replace')
+      self._log += logd
+      if logd.count('ERROR:') > 0:
+         warnings.warn("Noticed 'ERROR:' in LOG, you ought to take a look and see if there was a problem")
+         self._sb.check_error_log = True
+ 
       if k_dts is None:
          dts = {}
          for i in range(nvars):
